# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools import config
import time
from tools.translate import _
import netsvc

class account_payment_term(osv.osv):
    _name = "account.payment.term"
    _inherit = "account.payment.term"
    _columns = {
        'cash_discount_ids': fields.one2many('account.cash.discount', 'payment_id', 'Cash Discount Terms'),
    }
account_payment_term()

class account_cash_discount(osv.osv):
    _name = "account.cash.discount"
    _description = "Cash Discount"
    _order ="delay"
    _columns = {
        'name': fields.char('Name', size=32),
        'delay': fields.integer('Number of Days', required=True),
        'discount': fields.float('Discount (%)', digits=(16,2), required=True),
        'discount_account_id': fields.many2one('account.account', 'Customer Discount Account', required=True),
        'sup_discount_account_id': fields.many2one('account.account', 'Supplier Discount Account', required=True),
        'payment_id': fields.many2one('account.payment.term', 'Related Payment Term'),
    }
account_cash_discount()

class account_payment_term_line(osv.osv):
    _inherit = "account.payment.term.line"
    _columns = {
        'day_tolerance': fields.integer('Tolerance (Days)', digits=(16,0)),
        'payment_id': fields.many2one('account.payment.term', 'Payment Term', required=False, select=True),
        'compl_cash_discount': fields.boolean('Use Complete Cash Discount'),
    }
    _defaults = {
        'day_tolerance': lambda *a: 0,
        }
account_payment_term_line()

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def _get_amount(self, cr, uid, ids, residual_amount, payment_term, context=None):
        """
        This function returns the amount to be paid according to the cash discount payment term for the corresponding payment term 
        """
        if context is None:
           context = {}
        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        date1 = invoice.date_invoice or time.strftime('%Y-%m-%d')
        date2 = 'date_p' in context and context['date_p'] or time.strftime('%Y-%m-%d')
        from_dt = time.mktime(time.strptime(date1, '%Y-%m-%d'))
        to_dt = time.mktime(time.strptime(date2, '%Y-%m-%d'))
        diff_day = abs(from_dt - to_dt) / (3600 * 24)

        discount = amount = tol_days = 0.0
        payment_line_found = False

        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.line_ids:
                    res = dis = 0.0
                    for payment_line in payment_term_lines.line_ids:
                        if diff_day >= dis and diff_day <= payment_line.days:
                            if residual_amount:
                                payment_line_found = True
                                tol_days = payment_line.day_tolerance
                                if payment_line.value == 'procent':
                                    res = round((residual_amount * payment_line.value_amount), int(config['price_accuracy']))
                                elif payment_line.value == 'fixed':
                                    res = payment_line.value_amount
                                else:
                                    res = residual_amount
                            amount = res
                        dis = payment_line.days
        else:
            amount = residual_amount
                        
        return {'amount': amount, 'tol_days': tol_days, 'found': payment_line_found}

    def _get_payment(self, cr, uid, ids, residual_amount, payment_term, context=None):
        """
        This function returns the cash discount according to the cash discount payment term for the corresponding payment term
        """
        if context is None:
           context = {}

        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        date1 = invoice.date_invoice or time.strftime('%Y-%m-%d')
        date2 = 'date_p' in context and context['date_p'] or time.strftime('%Y-%m-%d')
        from_dt = time.mktime(time.strptime(date1, '%Y-%m-%d'))
        to_dt = time.mktime(time.strptime(date2, '%Y-%m-%d'))
        diff_day = abs(from_dt - to_dt) / (3600 * 24)
        tolerance = self._get_amount(cr, uid, ids, residual_amount, payment_term, context)
        tolerance_days = tolerance['tol_days']

        discount = 0.0
        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.cash_discount_ids:
                    res = dis = 0.0
                    for discount_line in payment_term_lines.cash_discount_ids:
                        if diff_day >= dis and diff_day <= discount_line.delay + tolerance_days:
                            if residual_amount:
                                res = round((residual_amount * discount_line.discount / 100), int(config['price_accuracy']))
                            discount = res
                        dis = discount_line.delay + tolerance_days
        return discount

    def _get_account(self, cr, uid, ids, residual_amount, payment_term, context=None):
        """
        This function returns the account according to the cash discount payment term for the corresponding payment term
        """
        if context is None:
           context = {}

        account_id = False

        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        date1 = invoice.date_invoice or time.strftime('%Y-%m-%d')
        date2 = 'date_p' in context and context['date_p'] or time.strftime('%Y-%m-%d')
        from_dt = time.mktime(time.strptime(date1, '%Y-%m-%d'))
        to_dt = time.mktime(time.strptime(date2, '%Y-%m-%d'))
        diff_day = abs(from_dt - to_dt) / (3600 * 24)
        tolerance = self._get_amount(cr, uid, ids, residual_amount, payment_term, context)
        tolerance_days = tolerance['tol_days']

        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.cash_discount_ids:
                    res = dis = 0.0
                    for discount_line in payment_term_lines.cash_discount_ids:
                        if diff_day >= dis and diff_day <= discount_line.delay + tolerance_days:
                            if invoice.type in ('in_invoice', 'in_refund'):
                                account_id = discount_line.sup_discount_account_id.id
                            else:
                                account_id = discount_line.discount_account_id.id
                        dis = discount_line.delay + tolerance_days
                    return account_id

    def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context=None, name=''):
        if context is None:
            context = {}
        #TODO check if we can use different period for payment and the writeoff line
        assert len(ids) == 1, _("Can only pay one invoice at a time!")
        invoice = self.browse(cr, uid, ids[0])

        invoice_tax_obj = self.pool.get("account.invoice.tax")
        journal_obj = self.pool.get('account.journal')
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        move_line_obj_memory = self.pool.get('account.move.line.memory')
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.user')

        # get discount from payment term if any
        amount_discount = invoice.payment_term and self._get_payment(cr, uid, ids, pay_amount, invoice.payment_term.id, context=context) or 0.0
        journal_currency = journal_obj.browse(cr, uid, pay_journal_id, context=context).currency.id
         
        if not name:
            name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number
        src_account_id = invoice.account_id.id
        # Take the seq as name for move
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        #take the choosen date
        date = 'date_p' in context and context['date_p'] or time.strftime('%Y-%m-%d')

        # Take the amount in currency and the currency of the payment
        if 'amount_currency' in context and context['amount_currency'] and 'currency_id' in context and context['currency_id']:
            amount_currency = context['amount_currency']
            currency_id = context['currency_id']
        else:
            amount_currency = False
            currency_id = False
            
        if invoice.type in ('in_invoice', 'in_refund'):
            ref = invoice.reference
        else:
            ref = self._convert_ref(cr, uid, invoice.number)

        discount_amount = tax_total_amount = 0.0
        if 'cash_amount' in context and context['cash_amount']:
            discount_amount = context['cash_amount']
        elif 'pay_cash_discount' in context and context['pay_cash_discount']:
            discount_amount = context['pay_cash_discount']
            
        # Pay attention to the sign for both debit/credit AND amount_currency
        l1 = {
            'debit': direction * pay_amount > 0 and abs(pay_amount) or 0.0,
            'credit': direction * pay_amount < 0 and abs(pay_amount) or 0.0,
            'account_id': src_account_id,
            'partner_id': invoice.partner_id.id,
            'ref': ref,
            'date': date,
            'currency_id': currency_id,
            'amount_currency': amount_currency and direction * amount_currency or 0.0,
            'name': name,
        }

        ### if we change the amount paid and in the disocunt then it used the changes discount
        if amount_discount != discount_amount and discount_amount > 0.0:
            amount_discount = discount_amount
        l2 = {
            'debit': direction * (pay_amount - amount_discount) < 0 and abs(pay_amount - amount_discount) or 0.0,
            'credit': direction * (pay_amount - amount_discount) > 0 and abs(pay_amount - amount_discount) or 0.0,
            'account_id': pay_account_id,
            'partner_id': invoice.partner_id.id,
            'ref': ref,
            'date': date,
            'currency_id': currency_id,
            'amount_currency': amount_currency and -direction * amount_currency or 0.0,
            'name': name,
        }

        lines = [(0, 0, l1), (0, 0, l2)]

        ### When we make payment by pay invoice wizard and if found the taxes move then directly take the entry from the wizard
        if 'tax_move_ids' in context and context['tax_move_ids']:
            move_line = context['tax_move_ids']
            for move_line_id in move_line:
                move_line_data = move_line_obj_memory.browse(cr, uid,move_line_id)
                l3 = {
                    'debit': move_line_data.debit,
                    'credit': move_line_data.credit,
                    'account_id': move_line_data.account_id.id,
                    'partner_id': invoice.partner_id.id,
                    'ref': ref,
                    'date': move_line_data.date,
                    'currency_id': move_line_data.currency_id.id,
                    'amount_currency': move_line_data.amount_currency and direction * move_line_data.amount_currency or 0.0,
                    'name': move_line_data.name,
                    'tax_code_id': move_line_data.tax_code_id.id,
                    'tax_amount': move_line_data.tax_amount,
                }
                lines.append((0, 0, l3))
        else:
            if amount_discount > 0.0:
                for line in invoice.invoice_line:
                    line_tax_amount = 0.0
                    tax_account_id = tax_code_id = tax_base = tax_code = False
                    
                    for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal / line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                        line_tax_amount += tax['amount']
                    line_total = line.price_subtotal + line_tax_amount
                    invoice_res_amount = invoice.amount_total
                    line_ratio = line_total / invoice_res_amount
                    
                    if line.invoice_line_tax_id:
                        total_tax_amount = tax_amount_currency = 0.0
                        for tax_id in line.invoice_line_tax_id:
                            tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                            discount_ratio = amount_discount * line_ratio
                            tax_amt = round((discount_ratio * tax_data.amount), int(config['price_accuracy']))
                            tax_real_amt = round((tax_amt / (1 + tax_data.amount)), int(config['price_accuracy']))
                            total_tax_amount += tax_real_amt

                            tax_base = tax_data.base_code_id.id
                            tax_code = tax_data.tax_code_id.id  
                            tax_invoice_id = invoice_tax_obj.search(cr, uid, [('invoice_id', '=', invoice.id), ('tax_code_id', '=', tax_code), ('base_code_id', '=', tax_base)])
                        
                            for tax_id2 in tax_invoice_id:
                                tax_value = invoice_tax_obj.browse(cr, uid, tax_id2)
                                tax_account_id = tax_value.account_id.id
                                tax_code_id = tax_value.tax_code_id.id

                            if invoice.company_id.currency_id.id <> invoice.currency_id.id:
                                tax_amount_currency = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, tax_real_amt, context=context)
                            l3 = {
                                'debit': direction * tax_real_amt < 0 and abs(tax_real_amt) or 0.0,
                                'credit': direction * tax_real_amt > 0 and abs(tax_real_amt) or 0.0,
                                'account_id': tax_account_id,
                                'partner_id': invoice.partner_id.id,
                                'ref': ref,
                                'date': date,
                                'currency_id': currency_id,
                                'amount_currency': tax_amount_currency and direction * tax_amount_currency or 0.0,
                                'tax_code_id': tax_code_id,
                                'tax_amount': -tax_real_amt,
                                'name': name,
                            }
                            lines.append((0, 0, l3))
        ### When we make payment by pay invoice wizard and if found the discount moves in wizard discount move then directly take the entry from the wizard
        if 'discount_move_ids' in context and context['discount_move_ids']:
            move_line = context['discount_move_ids']
            for move_line_id in move_line:
                move_line_data = move_line_obj_memory.browse(cr, uid, move_line_id)
                l4 = {
                    'debit': move_line_data.debit,
                    'credit':move_line_data.credit,
                    'account_id': move_line_data.account_id.id,
                    'partner_id': invoice.partner_id.id,
                    'ref':ref,
                    'date': move_line_data.date,
                    'currency_id': move_line_data.currency_id.id,
                    'amount_currency': move_line_data.amount_currency and direction * move_line_data.amount_currency or 0.0,
                    'name': move_line_data.name,
                    'tax_code_id': move_line_data.tax_code_id.id,
                    'tax_amount': -move_line_data.tax_amount,
                }
                lines.append((0, 0, l4))
        else:
            if amount_discount > 0:
                account_id = False
                if 'account_id' in context and context['account_id']:
                    account_id = context['account_id']
                elif 'pay_cash_dis_acc' in context and context['pay_cash_dis_acc']:
                    account_id = context['pay_cash_dis_acc']
                else:
                    account_id = self._get_account(cr, uid, ids , pay_amount, invoice.payment_term.id, context=context)
                
                fpos = invoice.fiscal_position or False
                if account_id:
                    account_id = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, account_id)

                for line in invoice.invoice_line:
                    tax_account_id = base_code_id = False  
                    tax_base = tax_code = False
                    
                    discount_amount_currency = line_tax_amount = total_tax_amount = 0.0  
                    for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                        line_tax_amount += tax['amount']
                    line_total = line.price_subtotal + line_tax_amount
                    invoice_res_amount = invoice.amount_total
                    line_ratio = line_total / invoice_res_amount

                    discount_ratio = amount_discount * line_ratio
                    
                    for tax_id in line.invoice_line_tax_id:
                        tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                        tax_amt = round((discount_ratio * tax_data.amount), int(config['price_accuracy']))
                        tax_real_amt = round((tax_amt / (1 + tax_data.amount)), int(config['price_accuracy']))
                        total_tax_amount += tax_real_amt

                        tax_base = tax_data.base_code_id.id
                        tax_code = tax_data.tax_code_id.id
                        tax_invoice_id = invoice_tax_obj.search(cr, uid, [('invoice_id', '=', invoice.id), ('tax_code_id', '=', tax_code), ('base_code_id', '=', tax_base)])
                        
                        for tax_id2 in tax_invoice_id:
                            tax_value = invoice_tax_obj.browse(cr, uid, tax_id2)
                            tax_account_id = tax_value.account_id.id
                            base_code_id = tax_value.base_code_id.id

                    if invoice.company_id.currency_id.id <> invoice.currency_id.id:
                        discount_amount = discount_ratio - total_tax_amount
                        discount_amount_currency = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, discount_amount, context=context)
                    l4 = {
                        'debit': direction * (discount_ratio - total_tax_amount) < 0 and abs(discount_ratio - total_tax_amount) or 0.0,
                        'credit': direction * (discount_ratio - total_tax_amount) > 0 and abs(discount_ratio - total_tax_amount) or 0.0,
                        'account_id': account_id,
                        'partner_id': invoice.partner_id.id,
                        'ref': ref,
                        'date': date,
                        'currency_id': currency_id,
                        'amount_currency': discount_amount_currency and direction * discount_amount_currency or 0.0,
                        'tax_code_id': base_code_id,
                        'tax_amount': -(discount_ratio - total_tax_amount),
                        'name': name,
                    }
                    lines.append((0, 0, l4))
        
        ########################### for currency difference#################
        currency_diff = invoice_amount = payment_line_amount = 0.0
        
        for line in invoice.move_id.line_id:
            move_line_data = move_line_obj.browse(cr, uid, line.id, context=context)
            invoice_amount += line.debit
        
        for payment_line in invoice.payment_ids:
            if payment_line.debit > 0.0:
                payment_line_amount += payment_line.debit 
            elif payment_line.credit > 0.0:
                payment_line_amount += payment_line.credit
                
        payment_line_amount += pay_amount

        if journal_currency != invoice.currency_id.id or invoice.currency_id.id != invoice.company_id.currency_id.id:
            currency_diff = invoice_amount - payment_line_amount
            expense_acc = invoice.company_id.expense_account_id
            revenue_acc = invoice.company_id.revenue_account_id
            partner_acc_id = curr_diff_account_id = False
            ## Account selection############
            if currency_diff < 0.00:
                curr_diff_account_id = expense_acc.id
            elif currency_diff > 0.00:
                curr_diff_account_id = revenue_acc.id
            ###### set fiscal position for the currency_diff account
            fpos = invoice.fiscal_position or False
            if curr_diff_account_id:
                curr_diff_account_id = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, curr_diff_account_id)

            if currency_diff < 0.00 and invoice.type in ['in_invoice','out_refund'] or currency_diff > 0.00 and invoice.type in ['out_invoice','in_refund']:
                l5 = {
                    'ref': ref,
                    'date': date,
                    'partner_id': invoice.partner_id.id,
                    'account_id': invoice.account_id.id,
                    'credit': abs(currency_diff),
                    'debit': 0.0,
                    'amount_currency': 0.0,
                    'name': name,
                }
                lines.append((0, 0, l5))
                
                l6 = {
                    'ref': ref,
                    'date': date,
                    'partner_id': invoice.partner_id.id,
                    'account_id': curr_diff_account_id,
                    'credit': 0.0,
                    'debit': abs(currency_diff),
                    'amount_currency': 0.0,
                    'name': name,
                }
                lines.append((0, 0, l6))
            
            if currency_diff > 0.00 and invoice.type in ['in_invoice','out_refund'] or currency_diff < 0.00 and invoice.type in ['out_invoice','in_refund']:
                l5 = {
                    'ref': ref,
                    'date': date,
                    'partner_id': invoice.partner_id.id,
                    'account_id': invoice.account_id.id,
                    'credit': 0.0,
                    'debit': abs(currency_diff),
                    'amount_currency': 0.0,
                    'name': name,
                }
                lines.append((0, 0, l5))
                
                l6 = {
                    'ref': ref,
                    'date': date,
                    'partner_id': invoice.partner_id.id,
                    'account_id': curr_diff_account_id,
                    'credit': abs(currency_diff),
                    'debit': 0.0,
                    'amount_currency': 0.0,
                    'name': name,
                }
                lines.append((0, 0, l6))
                
        move = {'ref': ref, 'line_id': lines, 'journal_id': pay_journal_id, 'period_id': period_id, 'date': date}
        move_id = move_obj.create(cr, uid, move, context=context)

        line_ids = []
        total = 0.0
        line = self.pool.get('account.move.line')
        cr.execute('SELECT id FROM account_move_line WHERE move_id IN (%s, %s)', (move_id, invoice.move_id.id,))

        lines = line.browse(cr, uid, map(lambda x: x[0], cr.fetchall()) )
        for l in lines + invoice.payment_ids:
            if l.account_id.id == src_account_id:
                line_ids.append(l.id)
                total += (l.debit or 0.0) - (l.credit or 0.0)
        if (not round(total, int(config['price_accuracy']))) or writeoff_acc_id:
            self.pool.get('account.move.line').reconcile(cr, uid, line_ids, 'manual', writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
        else:
            self.pool.get('account.move.line').reconcile_partial(cr, uid, line_ids, 'manual', context)

        # Update the stored value (fields.function), so we write to trigger recompute
        self.pool.get('account.invoice').write(cr, uid, ids, {}, context=context)
        return True

account_invoice()

class account_bank_statement_reconcile(osv.osv):
    _inherit='account.bank.statement.reconcile'
    def name_get(self, cursor, user, ids, context=None):
        res = []
        res = super(account_bank_statement_reconcile, self).name_get(cursor, user, ids, context)   
        for o in self.browse(cursor, user, ids, context=context):
            data = res[0][1] + o.name
            res.append((o.id, data))
        return res
account_bank_statement_reconcile()    

class account_bank_statement_reconcile_line(osv.osv):
    _inherit='account.bank.statement.reconcile.line'
    _columns = {
        'tax_code_id': fields.many2one('account.tax.code', 'Tax Account'),
    }
account_bank_statement_reconcile_line()

class res_company(osv.osv):
    _inherit  = "res.company"
    _columns = {
        'expense_account_id': fields.many2one('account.account', 'Account for Expense from Exchange Rate Difference', required=True),
        'revenue_account_id': fields.many2one('account.account', 'Account for Revenue from Exchange Rate Difference', required=True),
    }
res_company()

class res_company_wizard(osv.osv_memory):
    _name = 'res.company.wizard'
    
    _columns = {
        'expense_account_id': fields.many2one('account.account', 'Account for Expense from Exchange Rate Difference', required=True),
        'revenue_account_id': fields.many2one('account.account', 'Account for Revenue from Exchange Rate Difference', required=True),
    }

    def set_default(self, cr, uid, ids, context=None):
        company_id = self.pool.get('res.users').browse(cr,uid,[uid],context)[0].company_id
        for o in self.browse(cr, uid, ids, context=context):
             self.pool.get('res.company').write(cr, uid, [company_id.id], {'expense_account_id': o.expense_account_id.id, 'revenue_account_id': o.revenue_account_id.id})
        return {
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'ir.actions.configuration.wizard',
            'type': 'ir.actions.act_window',
            'target': 'new',
         }

    def action_cancel(self, cr, uid, ids, context=None):
        return {
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'ir.actions.configuration.wizard',
            'type': 'ir.actions.act_window',
            'target': 'new',
        }

res_company_wizard()

class account_bank_statement(osv.osv):
    _inherit="account.bank.statement"

    def _end_balance(self, cr, uid, ids, name, attr, context=None):
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')
        tax_obj = self.pool.get('account.tax')
        res = {}

        statements = self.browse(cr, uid, ids, context=context)
        for statement in statements:
            res[statement.id] = statement.balance_start
            currency_id = statement.currency.id
            for line in statement.move_line_ids:
                if line.debit > 0 and line.account_id.id == statement.journal_id.default_debit_account_id.id:
                    res[statement.id] += line.amount_currency or line.debit
                elif line.credit > 0 and line.account_id.id == statement.journal_id.default_credit_account_id.id:
                    res[statement.id] += line.amount_currency or -line.credit
                    
            if statement.state == 'draft':
                for line in statement.line_ids:
                    tax = 1
                    if line.type == 'general' and line.tax_type == 'tax_excluded' and line.tax_id:
                        tax = 1 + tax_obj.browse(cr, uid, line.tax_id.id, context=context).amount
                    res[statement.id] += line.amount * tax - line.cash_discount * tax

        for r in res:
            res[r] = round(res[r], int(config['price_accuracy']))
        return res

    _columns = {
        'balance_end': fields.function(_end_balance, method=True, string = 'Balance'),
    }
    
    def button_confirm(self, cr, uid, ids, context={}):
        done = []
        account_bank_statement_line_obj = self.pool.get('account.bank.statement.line')
        account_move_obj = self.pool.get('account.move')
        account_move_line_obj = self.pool.get('account.move.line')
        tax_obj = self.pool.get('account.tax')
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')
        
        company_currency_id = res_users_obj.browse(cr, uid, uid, context=context).company_id.currency_id.id
        expense_acc = res_users_obj.browse(cr, uid, uid, context=context).company_id.expense_account_id.id
        revenue_acc = res_users_obj.browse(cr, uid, uid, context=context).company_id.revenue_account_id.id
        
        for st in self.browse(cr, uid, ids, context):
            if not st.state == 'draft':
                continue
            if not (abs(st.balance_end - st.balance_end_real) < 0.0001):
                raise osv.except_osv(_('Error!'),
                        _('The statement balance is incorrect!\n') +
                        _('The expected balance (%.2f) is different than the computed one. (%.2f)') % (st.balance_end_real, st.balance_end))
            if (not st.journal_id.default_credit_account_id) or (not st.journal_id.default_debit_account_id):
                raise osv.except_osv(_('Configuration Error!'),
                    _('Please verify that an account is defined in the journal.'))
            currency_id = st.currency.id <> company_currency_id and st.currency.id
            
            for line in st.move_line_ids:
                if line.state <> 'valid':
                    raise osv.except_osv(_('Error!'),
                        _('The account entry lines are not in valid state.'))
            # for bank.statement.lines
            # In line we get reconcile_id on bank.ste.rec.
            # in bank stat.rec we get line_new_ids on bank.stat.rec.line
            new_end_value = 0.0
            for move in st.line_ids:
                context.update({'date': move.date,})
                move_id = account_move_obj.create(cr, uid, {
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'date': move.date,
                }, context=context)
                account_bank_statement_line_obj.write(cr, uid, [move.id], {
                    'move_ids': [(4, move_id, False)]
                })
                if not move.amount:
                    continue
                torec = []
                account_id = move.amount >= 0 and st.journal_id.default_credit_account_id.id or st.journal_id.default_debit_account_id.id
                acc_cur = move.amount <= 0 and st.journal_id.default_debit_account_id or move.account_id
                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                    company_currency_id, move.amount, context=context,
                    account = acc_cur)

                st_tax_amount = move_tax_amount = tax_real_amt = 0.0
                st_base_code_id = curr_diff_account_id = False
                
                if move.type == 'general' and move.tax_type <> 'untaxed':
                    if move.tax_type == 'tax_included':
                        st_tax_data = tax_obj.browse(cr, uid, move.tax_id.id, context=context)
                        st_tax_amt = move.amount * st_tax_data.amount
                        st_tax_amount = st_tax_amt / (1 + st_tax_data.amount)
                        st_base_code_id = st_tax_data.base_code_id.id
                        amount = amount - st_tax_amount
                        tax_pay = st_tax_amount
                        st_tax_amount = 0.0
                    else:
                        st_tax_data = tax_obj.browse(cr, uid, move.tax_id.id, context=context)
                        st_tax_amt = move.amount * st_tax_data.amount
                        st_tax_amount = st_tax_amt
                        tax_pay = st_tax_amount
                        st_base_code_id = st_tax_data.base_code_id.id
                    
                    if not st_tax_data.tax_code_id.id:
                        tax_val = {
                            'name': move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': move.partner_id and move.partner_id.id,
                            'account_id': move.account_id and move.account_id.id,
                            'credit': tax_pay > 0 and abs(tax_pay) or 0.0,
                            'debit': tax_pay < 0 and abs(tax_pay) or 0.0,
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                            'currency_id': currency_id,
                            'tax_code_id': st_base_code_id,
                            'tax_amount': move_tax_amount,
                        }
                        torec.append(account_move_line_obj.create(cr, uid, tax_val, context=context))    
                
                val = {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': move.partner_id and move.partner_id.id,
                    'account_id': move.account_id and move.account_id.id,
                    'credit': amount > 0 and abs(amount) or 0.0,
                    'debit': amount < 0 and abs(amount) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'currency_id': currency_id,
                    'tax_code_id': st_base_code_id,
                    'account_tax_id': move.tax_id.id,
                    'tax_amount': move_tax_amount,
                }
                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                    company_currency_id, move.amount, context=context,
                    account=acc_cur)
                val['amount_currency'] = currency_id and -move.amount or 0.00

                if move.account_id.currency_id and \
                        st.currency.id <> company_currency_id <> move.account_id.currency_id.id <> st.currency_id:
                    
                    val['currency_id'] = move.account_id.currency_id.id
                    val['amount_currency'] = res_currency_obj.compute(cr, uid, company_currency_id,
                                    move.account_id.currency_id.id, -amount, context=context, account = acc_cur)

                torec.append(account_move_line_obj.create(cr, uid, val, context=context))
                
                newline_sum = 0.0
                if move.reconcile_id and move.reconcile_id.line_new_ids:
                    newline_amount_currency = 0.0
                    for newline in move.reconcile_id.line_new_ids:
                        newline_amount_currency = res_currency_obj.compute(cr, uid, st.currency.id,
                                                        company_currency_id, newline.amount, context=context)
                        newline_amount_currency = round(newline_amount_currency, int(config['price_accuracy'])) 
                        newline_sum += newline_amount_currency

                        account_move_line_obj.create(cr, uid, {
                            'name': newline.name or move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': move.partner_id and move.partner_id.id,
                            'account_id': newline.account_id and newline.account_id.id,
                            'debit': newline_amount_currency > 0 and abs(newline_amount_currency) or 0.0,
                            'credit': newline_amount_currency < 0 and abs(newline_amount_currency) or 0.0,
                            'statement_id': st.id,
                            'tax_code_id': newline.tax_code_id.id,
                            'tax_amount': -newline_amount_currency,
                            'amount_currency': newline.amount,
                            'currency_id': currency_id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                        }, context=context)

                amount_currency = False
                if move.reconcile_id and move.reconcile_id.line_ids:
                    for line in move.reconcile_id.line_ids:
                        move_line_data = self.pool.get('account.move.line').browse(cr, uid, line.id, context=context)
                        if st.currency.id <> company_currency_id:
                            balance = reconcile_sum = 0.0
                            context.update({'amount': move_line_data.amount_currency, 'currency_id': move_line_data.currency_id.id  or company_currency_id})
                            reconcile_data = self.pool.get("account.bank.statement.reconcile").browse(cr, uid, move.reconcile_id.id, context=context)
                            balance_sum = reconcile_data.total_balance
                            for line_reconcile in reconcile_data.line_new_ids:
                                reconcile_sum += line_reconcile.amount
                            balance = round((balance_sum - reconcile_sum), int(config['price_accuracy']))
                            
                            if st.currency.id <> move_line_data.currency_id.id:
                                invoice_cret_balance = 0.0 
                                payment_balance = 0.0
                                context.update({'date': move_line_data.date})
                                inv_create_balance = res_currency_obj.compute(cr, uid, company_currency_id,
                                                            st.currency.id, amount, context=context)
                                currenct_date = time.strftime('%Y-%m-%d')
                                context.update({'date': currenct_date})
                                payment_balance = res_currency_obj.compute(cr, uid, company_currency_id,
                                                        st.currency.id, amount, context=context)
                                balance = round(inv_create_balance - payment_balance, int(config['price_accuracy']))
    
                            ## Account selection############
                            if balance < 0.00:
                                curr_diff_account_id = expense_acc
                            elif balance > 0.00:
                                curr_diff_account_id = revenue_acc
                            #######################################
                            
                            ###### set fiscal position for the currency_diff account
                            fpos = move_line_data.invoice.fiscal_position or False
                            if curr_diff_account_id:
                                curr_diff_account_id = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, curr_diff_account_id)
                            if balance <> 0.0:
                                currency_diff1 = account_move_line_obj.create(cr, uid, {
                                    'name': move.name,
                                    'date': move.date,
                                    'ref': move.ref,
                                    'move_id': move_id,
                                    'partner_id': move.partner_id and move.partner_id.id,
                                    'account_id': move.account_id.id,
                                    'credit': balance < 0.0 and abs(balance) or 0.0,
                                    'debit': balance > 0.0 and abs(balance) or 0.0,
                                    'statement_id': st.id,
                                    'journal_id': st.journal_id.id,
                                    'period_id': st.period_id.id,
                                    'amount_currency': 0.0,
                                    'currency_id': currency_id,
                                }, context=context)
    
                                currency_diff2 = account_move_line_obj.create(cr, uid, {
                                    'name': move.name,
                                    'date': move.date,
                                    'ref': move.ref,
                                    'move_id': move_id,
                                    'partner_id': move.partner_id and move.partner_id.id,
                                    'account_id': curr_diff_account_id,
                                    'credit': balance > 0.0 and abs(balance) or 0.0,
                                    'debit': balance < 0.0 and abs(balance) or 0.0,
                                    'statement_id': st.id,
                                    'journal_id': st.journal_id.id,
                                    'period_id': st.period_id.id,
                                    'amount_currency': 0.0,
                                    'currency_id': currency_id,
                                }, context=context)
                                
                                if move_line_data.invoice.currency_id.id != company_currency_id:
                                    torec.append(currency_diff1)

                if currency_id:
                    amount_currency = res_currency_obj.compute(cr, uid, company_currency_id, st.currency.id,
                        (amount + st_tax_amount - newline_sum), context=context, account = acc_cur)
                
                account_move_line_obj.create(cr, uid, {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': move.partner_id and move.partner_id.id,
                    'account_id': account_id,
                    'credit': (amount + st_tax_amount - newline_sum) < 0 and abs(amount + st_tax_amount - newline_sum) or 0.0,
                    'debit': (amount + st_tax_amount - newline_sum) > 0 and abs(amount + st_tax_amount - newline_sum) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'amount_currency': amount_currency,
                    'currency_id': currency_id,
                    }, context=context)

                account_move_line_obj.write(cr, uid, [x.id for x in
                        account_move_obj.browse(cr, uid, move_id, context=context).line_id], {'statement_id': st.id,})
                for line in account_move_line_obj.browse(cr, uid, [x.id for x in
                        account_move_obj.browse(cr, uid, move_id, context=context).line_id], context=context):
                    if line.state <> 'valid':
                        raise osv.except_osv(_('Error!'),
                                _('Account move line "%s" is not valid') % line.name)
                if move.reconcile_id and move.reconcile_id.line_ids:
                    torec += map(lambda x: x.id, move.reconcile_id.line_ids)
                    #try:
                    if abs(move.reconcile_amount - move.amount) < 0.0001:
                        writeoff_acc_id = False
                        #There should only be one write-off account!
                        for entry in move.reconcile_id.line_new_ids:
                            writeoff_acc_id = entry.account_id.id
                            break
                        account_move_line_obj.reconcile(cr, uid, torec, 'statement', writeoff_acc_id=writeoff_acc_id, writeoff_period_id=st.period_id.id, writeoff_journal_id=st.journal_id.id, context=context)
                    else:
                        account_move_line_obj.reconcile_partial(cr, uid, torec, 'statement', context)
                    #except:
                    #    raise osv.except_osv(_('Error!'), _('Unable to reconcile entry "%s": %.2f') % (move.name, move.amount))

                if st.journal_id.entry_posted:
                    account_move_obj.write(cr, uid, [move_id], {'state': 'posted'})
            done.append(st.id)
        
        self.write(cr, uid, done, {'state': 'confirm'}, context=context)
        return True

account_bank_statement()

class account_bank_statement_line(osv.osv):
    _inherit="account.bank.statement.line"
    _columns = {
        'tax_id': fields.many2one('account.tax', 'Tax'),
        'tax_type': fields.selection([
            ('untaxed', 'Untaxed'),
            ('tax_included', 'Tax included'),
            ('tax_excluded', 'Tax excluded')], 'Tax method', required=True,),
        'cash_discount': fields.float('Cash Discount'),
        'transferred_amount': fields.float('Transferred'),
    }

    def onchange_account_id(self, cr, uid, ids, account_id, context=None):
        result = {}
        tax = False
        taxes = self.pool.get('account.account').browse(cr, uid, account_id, context=context).tax_ids
        for tax_id in taxes:
            tax = tax_id.id
        result['tax_id'] = tax
        return {'value': result}

    def onchange_cash_discount(self, cr, uid, ids, amount, cash_discount, context=None):
        return {'value': {'transferred_amount': amount - cash_discount}}
    
    def onchange_transferred_amount(self, cr, uid, ids, amount, transferred_amount, context=None):
        return {'value': {'cash_discount': amount - transferred_amount}}

    def onchange_type(self, cr, uid, ids, type, context=None):
        result={}
        tax_id = self.pool.get('account.tax').search(cr, uid, [])

        if type == 'general' and tax_id:
           result['tax_id'] = tax_id[0]
        else:
            result['tax_id'] =  False
        return {'value': result}
    
    _defaults = {
        'tax_type':lambda *a: 'untaxed',
    }
account_bank_statement_line()


class account_move_line(osv.osv):
    _inherit="account.move.line"
    
    def reconcile_discount(self, cr, uid, ids, type='auto', discount_acc_id=False, discount_period_id=False, discount_journal_id=False, context={}):
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')
        partner_obj = self.pool.get('res.partner')
        res_users_obj = self.pool.get('res.users')

        id_set = ','.join(map(str, ids))
        reconcile_lines = []
        credit = debit = currency = 0.0
        currency_id = account_id = partner_id = invoice = ref = False
        discount_account = curr_diff_account = False
        if discount_acc_id:
            discount_account = self.pool.get('account.account').browse(cr, uid, discount_acc_id, context=context)
        lines = self.browse(cr, uid, ids, context=context)
        unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
        
        for line in unrec_lines:
            if line.invoice:
                invoice = line.invoice
            if line.state <> 'valid':
                raise osv.except_osv(_('Error!'),
                        _('Entry "%s" is not valid !') % line.name)
            credit += line['credit']
            debit += line['debit']
            currency += line['amount_currency'] or 0.0
            currency_id = currency_id or line['currency_id']
            account_id = line['account_id']['id']
            account_type = line['account_id']['type']
            partner_id = line['partner_id'] and line['partner_id']['id']
            ref = ref or line['ref']

        # simple rules for (full) reconciliation w/ cash discount:
        # if there's foreign currency involved:
        # - difference in foreign currency is cash discount
        # - difference in company currency is currency exchange rate difference
        # - negative difference is expense, positive difference is revenue
        # if there's no foreign currency:
        # - difference in foreign currency is 0.00
        # - difference in company currency is cash discount
        balance = credit - debit
        # take fiscal position from invoice, from partner as fallback
        fpos = invoice and invoice.fiscal_position
        if not fpos and partner_id:
            fpos = partner_obj.browse(cr, uid, partner_id, context=context).property_account_position
        date = context.has_key('date_p') and context['date_p'] or time.strftime('%Y-%m-%d')

        cr.execute('SELECT account_id, reconcile_id '\
                   'FROM account_move_line '\
                   'WHERE id IN %s '\
                   'GROUP BY account_id,reconcile_id',
                   (tuple(ids),))
        r = cr.fetchall()
#TODO: move this check to a constraint in the account_move_reconcile object
        if (len(r) != 1) and not context.get('fy_closing', False):
            raise osv.except_osv(_('Error!'), _('Entries are not of the same account or already reconciled!'))
        if not unrec_lines:
            raise osv.except_osv(_('Error!'), _('Entry is already reconciled!'))
        account = self.pool.get('account.account').browse(cr, uid, account_id, context=context)
        if not context.get('fy_closing', False) and not account.reconcile:
            raise osv.except_osv(_('Error!'), _('The account is not configured to be reconciled!'))
        if r[0][1] != None:
            raise osv.except_osv(_('Error!'), _('Some entries are already reconciled!'))

        # foreign currency
        if currency_id:
            # cash discount
            if not cur_obj.is_zero(cr, uid, currency_id, currency):
                if currency > 0:
                    osv.except_osv(_('Error!'), _('There seems to be a surcharge, not a cash discount. Can\'t reconcile this!'))
                if not discount_acc_id:
                    raise osv.except_osv(_('Error!'), _('You have to provide a cash discount account!'))
                if fpos:
                    discount_acc_id = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, discount_acc_id)
                # TODO: Check if cash discount amount in company currency should be computed with current date
                #       or date of last payment or date of invoice
                company_amount = round(cur_obj.compute(cr, uid, currency_id.id, account.company_id.currency_id.id, currency),
                    int(config['price_accuracy']))
                reconcile_lines.append(
                    (0, 0, {
                        'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                        'ref': ref,
                        'debit': abs(company_amount),
                        'credit': 0.0,
                        'account_id': account_id,
                        'date': date,
                        'partner_id': partner_id,
                        'currency_id': currency_id.id,
                        'amount_currency': -currency,
                    })
                )
                reconcile_lines.append(
                    (0, 0, {
                        'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                        'ref': ref,
                        'debit': 0.0,
                        'credit': abs(company_amount),
                        'account_id': discount_acc_id,
                        'analytic_account_id': context.get('analytic_id', False),
                        'date': date,
                        'partner_id': partner_id,
                        'currency_id': currency_id.id,
                        'amount_currency': currency,
                    })
                )
                # now the company currency difference needs to be corrected by the cash discount amount
                balance += company_amount

            # currency difference
            if not cur_obj.is_zero(cr, uid, account.company_id.currency_id, balance):
                if balance < 0.0:
                    curr_diff_account = res_users_obj.browse(cr, uid, uid, context=context).company_id.expense_account_id.id
                elif balance > 0.0:
                    curr_diff_account = res_users_obj.browse(cr, uid, uid, context=context).company_id.revenue_account_id.id
                if fpos:
                    curr_diff_account = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, curr_diff_account)
                reconcile_lines.append(
                    (0, 0, {
                        'name': invoice and invoice.number or balance < 0.0 and _('Currency Expense') or _('Currency Revenue'),
                        'ref': ref,
                        'debit': balance < 0.0 and abs(balance) or 0.0,
                        'credit': balance > 0.0 and abs(balance) or 0.0,
                        'account_id': curr_diff_account,
                        'analytic_account_id': context.get('analytic_id', False),
                        'date': date,
                        'partner_id': partner_id,
                        'currency_id': currency_id.id,
                        'amount_currency': 0.0,
                    })
                )
                reconcile_lines.append(
                    (0, 0, {
                        'name': invoice and invoice.number or balance < 0.0 and _('Currency Expense') or _('Currency Revenue'),
                        'ref': ref,
                        'debit': balance > 0.0 and abs(balance) or 0.0,
                        'credit': balance < 0.0 and abs(balance) or 0.0,
                        'account_id': account_id,
                        'date': date,
                        'partner_id': partner_id,
                        'currency_id': currency_id.id,
                        'amount_currency': 0.0,
                    })
                )
        # no foreign currency
        else:
            # cash discount
            if not cur_obj.is_zero(cr, uid, account.company_id.currency_id, balance):
                if balance > 0:
                    osv.except_osv(_('Error!'), _('There seems to be a surcharge, not a cash discount. Can\'t reconcile this!'))
                if not discount_acc_id:
                    raise osv.except_osv(_('Error!'), _('You have to provide a cash discount account!'))
                if fpos:
                    discount_acc_id = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, discount_acc_id)
                reconcile_lines.append(
                    (0, 0, {
                        'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                        'ref': ref,
                        'debit': balance > 0.0 and abs(balance) or 0.0,
                        'credit': balance < 0.0 and abs(balance) or 0.0,
                        'account_id': account_id,
                        'date': date,
                        'partner_id': partner_id,
                    })
                )
                # take care of taxes for the cash discount
                # cash discount -> base code
                # tax amount -> tax code
                taxed_balance = balance
                tax_amount = 0.0
                type = False
                # guess the type of payment
                if account.type == 'receivable':
                    type = balance > 0.0 and 'invoice' or balance < 0.0 and 'refund'
                elif account.type == 'payable':
                    type = balance < 0.0 and 'invoice' or balance > 0.0 and 'refund'
                if discount_account.tax_ids:
                    for tax in discount_account.tax_ids:
                        if tax.child_depend:
                            for ctax in tax.child_ids:
                                if not type or type == 'invoice':
                                    tax_amount = round(ctax.tax_sign * ctax.amount * balance,
                                        int(config['price_accuracy']))
                                else:
                                    tax_amount = round(ctax.ref_tax_sign * ctax.amount * balance,
                                        int(config['price_accuracy']))
                                reconcile_lines.append(
                                    (0, 0, {
                                        'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                                        'ref': ref,
                                        'debit': balance < 0.0 and abs(tax_amount) or 0.0,
                                        'credit': balance > 0.0 and abs(tax_amount) or 0.0,
                                        'account_id': type == 'invoice' and ctax.account_collected_id.id or type == 'refund' and ctax.account_paid_id.id,
                                        'date': date,
                                        'partner_id': partner_id,
                                        'tax_code_id': type == 'invoice' and ctax.tax_code_id.id or type == 'refund' and ctax.ref_tax_code_id.id,
                                        'tax_amount': tax_amount,                                        
                                    })
                                )
                        else:
                            if not type or type == 'invoice':
                                tax_amount = round(tax.tax_sign * tax.amount * balance, int(config['price_accuracy']))
                            else:
                                tax_amount = round(tax.ref_tax_sign * tax.amount * balance, int(config['price_accuracy']))
                            reconcile_lines.append(
                                (0, 0, {
                                    'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                                    'ref': ref,
                                    'debit': balance < 0.0 and abs(tax_amount) or 0.0,
                                    'credit': balance > 0.0 and abs(tax_amount) or 0.0,
                                    'account_id': type == 'invoice' and tax.account_collected_id.id or type == 'refund' and tax.account_paid_id.id,
                                    'date': date,
                                    'partner_id': partner_id,
                                    'tax_code_id': type == 'invoice' and tax.tax_code_id.id or type == 'refund' and tax.ref_tax_code_id.id,
                                    'tax_amount': tax_amount,
                                })
                            )
                    taxed_balance -= tax_amount
                    # only post with debit/credit on the first tax. The other rows would be needed for
                    # tax reports but need to have debit/credit of 0.0. We just need to have an entry
                    # for the base tax amount.
                    first_tax = True
                    for tax in discount_account.tax_ids:
                        reconcile_lines.append(
                            (0, 0, {
                                'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                                'ref': ref,
                                'debit': first_tax and balance < 0.0 and abs(taxed_balance) or 0.0,
                                'credit': first_tax and balance > 0.0 and abs(taxed_balance) or 0.0,
                                'account_id': discount_acc_id,
                                'analytic_account_id': context.get('analytic_id', False),
                                'date': date,
                                'partner_id': partner_id,
                                'tax_code_id': type == 'invoice' and tax.base_code_id.id \
                                    or type == 'refund' and tax.ref_base_code_id.id,
                                'tax_amount': taxed_balance,
                            })
                        )
                        first_tax = False
                else:
                    reconcile_lines.append(
                        (0, 0, {
                            'name': invoice and invoice.number or 'comment' in context and context['comment'] or _('Cash Discount'),
                            'ref': ref,
                            'debit': balance < 0.0 and abs(taxed_balance) or 0.0,
                            'credit': balance > 0.0 and abs(taxed_balance) or 0.0,
                            'account_id': discount_acc_id,
                            'analytic_account_id': context.get('analytic_id', False),
                            'date': date,
                            'partner_id': partner_id,
                        })
                    )
        if len(reconcile_lines):
            reconcile_move_id = self.pool.get('account.move').create(cr, uid, {
                'period_id': discount_period_id,
                'journal_id': discount_journal_id,
                'date': date,
                'state': 'draft',
                'line_id': reconcile_lines,                
                })
            reconcile_line_ids = self.search(cr, uid, [('move_id', '=', reconcile_move_id), ('account_id', '=', account_id)])
            ids += reconcile_line_ids

        r_id = self.pool.get('account.move.reconcile').create(cr, uid, {
            'type': type,
            'line_id': map(lambda x: (4, x, False), ids),
            'line_partial_ids': map(lambda x: (3, x, False), ids)
        })
        wf_service = netsvc.LocalService("workflow")
        # the id of the move.reconcile is written in the move.line (self) by the create method above
        # because of the way the line_id are defined: (4, x, False)
        for id in ids:
            wf_service.trg_trigger(uid, 'account.move.line', id, cr)
        return r_id
    
account_move_line()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

