# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta
from mx import DateTime
from tools.translate import _

_writeoff_entry_calculation_form = '''<?xml version="1.0"?>
<form string="Calculation">
    <separator string="Calculation of period-wise Write-Off Amount" colspan="4"/>
    <field name="cash_discount_id"/>
    <field name="tax_id"/>
    <field name="period_id"/>
    <field name="journal_id"/>
</form>'''

_writeoff_entry_calculation_fields = {
    'cash_discount_id': {
        'string': 'Cash Discount Account',
        'type': 'many2one',
        'relation': 'account.account',
        'required': True,
    },
    'tax_id': {
        'string': 'Taxes',
        'type': 'many2one',
        'relation': 'account.tax',
        'required': True,
    },
    'period_id': {
        'string': 'Period',
        'type': 'many2one',
        'relation': 'account.period',
        'required': True,
    },
    'journal_id': {
        'string': 'Journal',
        'type': 'many2one',
        'relation': 'account.journal',
        'required': True,
    },
}

def _compute(self, cr, uid, data, context):
    
    pool = pooler.get_pool(cr.dbname)
    invoice_obj = pool.get('account.invoice')
    account_move_obj= pool.get('account.move')
    account_move_line_obj = pool.get('account.move.line')
    payment_term_obj = pool.get('account.payment.term')
    tax_obj = pool.get('account.tax')
    
    period_id = data['form']['period_id']
    tax_id = data['form']['tax_id']
    journal_id = data['form']['journal_id']
    cash_discount_id = data['form']['cash_discount_id']
    
    account_move_line_ids = account_move_line_obj.search(cr, uid, [('period_id', '=', period_id), ('account_id', '=', cash_discount_id)])
    date = time.strftime('%Y-%m-%d')

    discount_lines = []
    
    tax_account_id = dis_tax_code_id = discount_code_id = False

    for line in account_move_line_obj.browse(cr, uid, account_move_line_ids, context=context):
        tax_discount_amt = 0.0
        tax_discount_data = tax_obj.browse(cr, uid, tax_id, context=context)
        tax_account_id = tax_discount_data.account_collected_id and tax_discount_data.account_collected_id.id or tax_discount_data.tax_code_id.id
        discount_code_id = tax_discount_data.base_code_id.id
        dis_tax_code_id = tax_discount_data.tax_code_id.id
            
        if line.debit > 0.0:
            tax_discount_amt = 0.0
            tax_discount_amt = line.debit * tax_discount_data.amount
                 
            discount_lines.append(
                    (0, 0, {
                        'name': line.name,
                        'debit': 0.0,
                        'credit': tax_discount_amt,
                        'account_id': cash_discount_id,
                        'date': date,
                        'partner_id': line.partner_id.id,
                        'tax_amount': -tax_discount_amt,
                        'tax_code_id': discount_code_id,
                    })),
            discount_lines.append(
                    (0, 0, {
                        'name': line.name,
                        'debit': tax_discount_amt,
                        'credit': 0.0,
                        'account_id': tax_account_id,
                        'date': date,
                        'partner_id': line.partner_id.id,
                        'tax_amount': -tax_discount_amt,
                        'tax_code_id': dis_tax_code_id,
                    }))
        elif line.credit > 0.0:
            tax_discount_amt = 0.0
            tax_discount_amt = line.credit * tax_discount_data.amount

            discount_lines.append(
                    (0, 0, {
                        'name': line.name,
                        'debit': 0.0,
                        'credit': tax_discount_amt,
                        'account_id': tax_account_id,
                        'date': date,
                        'partner_id': line.partner_id.id,
                        'tax_amount': -tax_discount_amt,
                        'tax_code_id': dis_tax_code_id,
                    }))
            discount_lines.append(
                    (0, 0, {
                        'name': line.name,
                        'debit': tax_discount_amt,
                        'credit': 0.0,
                        'account_id': cash_discount_id,
                        'date': date,
                        'partner_id': line.partner_id.id,
                        'tax_amount': -tax_discount_amt,
                        'tax_code_id': discount_code_id,
                    })),

    discount_move_line = account_move_obj.create(cr, uid, 
            {
            'period_id': period_id,
            'journal_id': journal_id,
            'date': date,
            'line_id': discount_lines,
            })        
    
    return {
        'domain': [('id', '=', discount_move_line)],
        'name': _('Computed Entries'),
        'view_type': 'form',
        'view_mode': 'tree,form',
        'view_id': False,
        'res_model': 'account.move',
        'type': 'ir.actions.act_window',
    }

class wiz_period_writeoff_calculation(wizard.interface):
    states = {
        'init': {
            'actions': [],
            'result': {
                'type': 'form',
                'arch': _writeoff_entry_calculation_form,
                'fields': _writeoff_entry_calculation_fields,
                'state': [('calculate', 'Compute'), ('end', 'Cancel')],
            }
        },
        'calculate': {
            'actions': [],
            'result': {
                'type': 'action',
                'action': _compute,
                'state': 'end',
            }
        }
    }
wiz_period_writeoff_calculation('account_period_write_calculation')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
